var dispatcher = require('../index');

describe('Event Dispatcher', function() {
  describe('when there are no observers', function() {
    it('should dispatch an event without notifications', function() {
      dispatcher.dispatch('TEST_EVENT', {});
    });
  });

  describe('when register a function observer', function() {
    var observerFunction;

    beforeEach(function() {
      observerFunction = jest.fn();

      dispatcher.register(observerFunction, 'TEST_EVENT');
    });

    afterEach(function() {
      dispatcher.unregister(observerFunction);
    });

    it('should call function after dispatch', function() {
      var args = {test: 'ok'};
      dispatcher.dispatch('TEST_EVENT', args);

      expect(observerFunction).toBeCalledWith('TEST_EVENT', args);
    });

    describe('when another event is dispatched', function() {
      beforeEach(function() {
        dispatcher.register(observerFunction, 'ANOTHER_TEST_EVENT');
      });

      it('observer should not be called', function() {
        expect(observerFunction).not.toBeCalled();
      });
    });
  });

  describe('when register an object observer', function() {
    var observerObject;

    beforeEach(function() {
      observerObject = {};
      observerObject.update = jest.fn();

      dispatcher.register(observerObject, 'TEST_EVENT');
    });

    afterEach(function() {
      dispatcher.unregister(observerObject);
    });

    it('should call function after dispatch', function() {
      var args = {test: 'ok'};
      dispatcher.dispatch('TEST_EVENT', args);

      expect(observerObject.update).toBeCalledWith('TEST_EVENT', args);
    });
  });

  describe('when register a function for all events', function() {
    var observerFunction;

    beforeEach(function() {
      observerFunction = jest.fn();

      dispatcher.register(observerFunction);
    });

    afterEach(function() {
      dispatcher.unregister(observerFunction);
    });

    it('should be called with any event', function() {
      dispatcher.dispatch('TEST_EVENT', {});
      dispatcher.dispatch('ANOTHER_TEST_EVENT', {});

      expect(observerFunction.mock.calls.length).toEqual(2);
      expect(observerFunction.mock.calls[0]).toEqual(['TEST_EVENT', {}]);
      expect(observerFunction.mock.calls[1]).toEqual(['ANOTHER_TEST_EVENT', {}]);
    });
  });

  describe('when register a function multiple events', function() {
    var observerFunction;

    beforeEach(function() {
      observerFunction = jest.fn();

      dispatcher.register(observerFunction, ['TEST_EVENT', 'ANOTHER_TEST_EVENT']);
    });

    afterEach(function() {
      dispatcher.unregister(observerFunction);
    });

    it('should be called with any event', function() {
      dispatcher.dispatch('TEST_EVENT', {});
      dispatcher.dispatch('ANOTHER_TEST_EVENT', {});

      expect(observerFunction.mock.calls.length).toEqual(2);
      expect(observerFunction.mock.calls[0]).toEqual(['TEST_EVENT', {}]);
      expect(observerFunction.mock.calls[1]).toEqual(['ANOTHER_TEST_EVENT', {}]);
    });

    describe('when unregister for one of those events', function() {
      beforeEach(function() {
        dispatcher.unregister(observerFunction, 'TEST_EVENT');
      });

      it('should be called with any event', function() {
        dispatcher.dispatch('TEST_EVENT', {});
        dispatcher.dispatch('ANOTHER_TEST_EVENT', {});

        expect(observerFunction.mock.calls.length).toEqual(1);
        expect(observerFunction).toBeCalledWith('ANOTHER_TEST_EVENT', {});
      });
    });

    describe('when unregister an event without observer', function() {
      beforeEach(function() {
        dispatcher.unregister(undefined, 'TEST_EVENT');
      });

      it('it should call the observer', function() {
        dispatcher.dispatch('TEST_EVENT', {});

        expect(observerFunction).toHaveBeenCalled();
      });
    });

    describe('when call unregister with no params', function() {
      beforeEach(function() {
        dispatcher.unregister();
      });

      it('it should call the observer', function() {
        dispatcher.dispatch('TEST_EVENT', {});

        expect(observerFunction).toHaveBeenCalled();
      });
    });
  });

  describe('when multiples observers register to the same event', function() {
    var observerFunction1, observerFunction2;

    beforeEach(function() {
      observerFunction1 = jest.fn();
      observerFunction2 = jest.fn();

      dispatcher.register(observerFunction1, 'TEST_EVENT');
      dispatcher.register(observerFunction2, 'TEST_EVENT');
    });

    afterEach(function() {
      dispatcher.unregister(observerFunction1);
      dispatcher.unregister(observerFunction2);
    });

    it('should call two observers', function() {
      dispatcher.dispatch('TEST_EVENT', {});

      expect(observerFunction1).toBeCalledWith('TEST_EVENT', {});
      expect(observerFunction2).toBeCalledWith('TEST_EVENT', {});
    });
  });
});
